<?php
/**
 * Render widget
 *
 * @package   WP Grid Builder - Elementor
 * @author    Loïc Blascos
 * @copyright 2019-2023 Loïc Blascos
 */

namespace WP_Grid_Builder_Elementor\Includes;

use Elementor\Plugin as Elementor;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Render
 *
 * @class WP_Grid_Builder_Elementor\Includes\Render
 * @since 1.0.0
 */
final class Render {

	/**
	 * Provider instance
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var WP_Grid_Builder_Elementor\Includes\Providers
	 */
	protected $providers;

	/**
	 * Holds Elementor widgets
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $widgets = [];

	/**
	 * Holds Elementor popups
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $popups = [];

	/**
	 * Wheter is built with elementor
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var array
	 */
	protected $is_elementor = false;

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param Providers $providers Providers instance.
	 */
	public function __construct( $providers ) {

		$this->providers = $providers;

		add_action( 'elementor/widget/before_render_content', [ $this, 'maybe_handle' ] );
		add_filter( 'elementor/frontend/builder_content_data', [ $this, 'get_widgets' ], 10, 2 );
		add_filter( 'wp_grid_builder/frontend/register_scripts', [ $this, 'register_script' ] );
		add_filter( 'wp_grid_builder/frontend/localize_script', [ $this, 'unset_widgets' ] );

	}

	/**
	 * Check if we can handle widget rendering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $widget Elementor widget settings.
	 */
	public function maybe_handle( $widget ) {

		$this->is_elementor = true;

		if ( ! $this->has_widget( $widget ) ) {
			return;
		}

		$post_id  = Elementor::instance()->documents->get_current()->get_main_id();
		$provider = $this->providers->get( $widget, $post_id );

		if ( ! $provider ) {
			return;
		}

		$provider->render();

	}

	/**
	 * Get and store filterable widgets on render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $widgets Holds Elementor widgets.
	 * @return array
	 */
	public function get_widgets( $widgets ) {

		foreach ( $widgets as $widget ) {

			$this->get_documents( $widget );

			if ( isset( $widget['widgetType'] ) && 'wpgb-facet' === $widget['widgetType'] ) {

				if ( ! empty( $widget['settings']['widget_id'] ) ) {
					$this->widgets[ 'elementor-element-' . $widget['settings']['widget_id'] ] = true;
				} elseif ( ! empty( $widget['settings']['grid'] ) ) {
					$this->widgets[ $widget['settings']['grid'] ] = true;
				}
			}

			if ( ! empty( $widget['elements'] ) ) {
				$this->get_widgets( $widget['elements'] );
			}
		}

		return $widgets;

	}

	/**
	 * Get dynamic documents (popup) from widget.
	 *
	 * @since 1.2.3
	 * @access public
	 *
	 * @param array $widget Hold Elementor widget.
	 */
	public function get_documents( $widget ) {

		if ( ! isset( $widget['settings']['__dynamic__'] ) ) {
			return;
		}

		$tags = $widget['settings']['__dynamic__'];

		if ( ! is_array( $tags ) ) {
			return;
		}

		foreach ( $tags as $tag ) {

			preg_match( '/settings="(.*?(?="]))/', $tag, $matches );

			if ( ! $matches ) {
				continue;
			}

			$settings = json_decode( urldecode( $matches[1] ), true );

			if ( empty( $settings['popup'] ) ) {
				continue;
			}

			$document = Elementor::instance()->documents->get( $settings['popup'] );

			if ( ! $document || isset( $this->popups[ $settings['popup'] ] ) ) {
				continue;
			}

			$this->popups[ $settings['popup'] ] = true;

			$this->get_widgets( $document->get_elements_data() );

		}
	}

	/**
	 * Check if filterable widget exists
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param Widget_Base $widget Hold widget instance to filter.
	 * @return boolean
	 */
	public function has_widget( $widget ) {

		return isset( $this->widgets[ 'elementor-element-' . $widget->get_id() ] );

	}

	/**
	 * Register script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $scripts Holds script to register.
	 * @return array
	 */
	public function register_script( $scripts ) {

		if ( $this->is_elementor ) {

			$scripts[] = [
				'handle'  => 'wpgb-elementor',
				'source'  => WPGB_ELEMENTOR_URL . 'assets/js/elementor.js',
				'version' => WPGB_ELEMENTOR_VERSION,
			];
		}

		return $scripts;

	}

	/**
	 * Unset shadow widgets generated by Elementor facets
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $data Holds data to localize.
	 * @return array
	 */
	public function unset_widgets( $data ) {

		if ( empty( $data['shadowGrids'] ) ) {
			return $data;
		}

		$data['shadowGrids'] = array_values(
			array_filter(
				$data['shadowGrids'],
				function( $id ) {
					return strpos( $id, 'elementor-element-' ) === false;
				}
			)
		);

		return $data;

	}
}
