<?php
/**
 * Hooks
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Hooks
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Hooks
 * @since 1.0.0
 */
class Hooks {

	/**
	 * Holds callback
	 *
	 * @since 1.0.0
	 * @access private
	 * @var callable
	 */
	private $callback;

	/**
	 * Hook priority
	 *
	 * @since 1.0.0
	 * @access private
	 * @var integer
	 */
	private $priority;

	/**
	 * Number of accepted arguments
	 *
	 * @since 1.0.0
	 * @access private
	 * @var integer
	 */
	private $accepted_args;

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param callable $callback      The callback to be run when the filter is applied.
	 * @param integer  $priority      Optional. Used to specify the order in which the functions associated with a particular action are executed.
	 * @param integer  $accepted_args Optional. The number of arguments the function accepts. Default 1.
	 */
	public function __construct( $callback = '', $priority = 10, $accepted_args = 1 ) {

		$this->callback      = $callback;
		$this->priority      = $priority;
		$this->accepted_args = $accepted_args;

	}

	/**
	 * Add filter to only run once.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return boolean
	 */
	public function once() {

		remove_filter( current_filter(), [ $this, __FUNCTION__ ], $this->priority );
		return call_user_func_array( $this->callback, array_slice( func_get_args(), 0, $this->accepted_args ) );

	}
}

/**
 * Add filter to only run once.
 *
 * @since 1.0.0
 *
 * @param string   $tag           The name of the filter to hook the $function_to_add callback to.
 * @param callable $callback      The callback to be run when the filter is applied.
 * @param integer  $priority      Optional. Used to specify the order in which the functions associated with a particular action are executed.
 * @param integer  $accepted_args Optional. The number of arguments the function accepts. Default 1.
 *
 * @return true
 */
function add_filter_once( $tag, $callback, $priority = 10, $accepted_args = 1 ) {

	$instance = new Hooks( $callback, $priority, $accepted_args );
	return add_filter( $tag, [ $instance, 'once' ], $priority, $accepted_args );

}

/**
 * Add action to only run once.
 *
 * @since 1.0.0
 *
 * @param string   $tag           The name of the filter to hook the $function_to_add callback to.
 * @param callable $callback      The callback to be run when the filter is applied.
 * @param integer  $priority      Optional. Used to specify the order in which the functions associated with a particular action are executed.
 * @param integer  $accepted_args Optional. The number of arguments the function accepts. Default 1.
 *
 * @return true
 */
function add_action_once( $tag, $callback, $priority = 10, $accepted_args = 1 ) {

	return add_filter_once( $tag, $callback, $priority, $accepted_args );

}
