<?php
/**
 * Oxygen providers
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Providers
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Providers
 * @since 1.0.0
 */
final class Providers extends Observer {

	/**
	 * Holds default Oxygen providers
	 *
	 * @since 1.0.0
	 * @access public
	 * @var array
	 */
	public $defaults = [
		'oxy_posts_grid'   => __NAMESPACE__ . '\Providers\Easy_Posts',
		'oxy_dynamic_list' => __NAMESPACE__ . '\Providers\Repeater',
		'oxy-woo-products' => __NAMESPACE__ . '\Providers\Products_List',
	];

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		parent::__construct();

		add_filter( 'wp_grid_builder_oxygen/providers', [ $this, 'register' ] );

	}

	/**
	 * Add default providers
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $providers Holds providers.
	 * @return array
	 */
	public function register( $providers ) {

		return array_merge( $this->defaults, $providers );

	}

	/**
	 * Handle provider before render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string       $name Provider name.
	 * @param array|string $attr Provider attributes.
	 */
	public function maybe_handle( $name, $attr ) {

		$provider = $this->get_provider( $name );

		if ( ! $provider ) {
			return;
		}

		$element = $this->get_element( $attr );

		if ( ! $element ) {
			return;
		}

		$instance = $this->instantiate( $provider, $element );

		if ( ! $instance ) {
			return;
		}

		$instance->before_render();

	}

	/**
	 * Handle provider after render
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string       $content Provider content.
	 * @param string       $name    Provider name.
	 * @param array|string $attr    Provider attributes.
	 * @return string
	 */
	public function maybe_render( $content, $name, $attr ) {

		$provider = $this->get_provider( $name );

		if ( ! $provider ) {
			return $content;
		}

		$element = $this->get_element( $attr );

		if ( ! $element ) {
			return $content;
		}

		$instance = Container::get( $element['id'] );

		if ( ! $instance ) {
			return $content;
		}

		return $instance->render( $content );

	}

	/**
	 * Instantiate provider
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $provider Provider name.
	 * @param array  $element  Provider element.
	 * @return false|Container
	 */
	public function instantiate( $provider, $element ) {

		if ( ! apply_filters( 'wp_grid_builder_oxygen/handle_element', false, $element['id'] ) ) {
			return false;
		}

		$providers = apply_filters( 'wp_grid_builder_oxygen/providers', [] );
		$settings  = apply_filters( 'wp_grid_builder_oxygen/provider_settings', $this->get_settings( [] ), $element['id'] );

		return Container::add( $element['id'], new $providers[ $provider ]( $element, $settings ) );

	}

	/**
	 * Get provider name
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $name Provider name.
	 * @return false|string
	 */
	public function get_provider( $name ) {

		$providers = apply_filters( 'wp_grid_builder_oxygen/providers', [] );
		$provider  = preg_replace( '/_[0-9]*$/', '', $name );

		if ( ! isset( $providers[ $provider ] ) || ! class_exists( $providers[ $provider ] ) ) {
			return false;
		}

		return $provider;

	}

	/**
	 * Get element attributes
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array|string $attr Provider attributes.
	 * @return false|array
	 */
	public function get_element( $attr ) {

		if ( empty( $attr['ct_options'] ) ) {
			return false;
		}

		$element = json_decode( $attr['ct_options'], true );

		if ( empty( $element['ct_id'] ) ) {
			return false;
		}

		$element['id'] = 'oxygen-element-' . $element['ct_id'];

		return $element;

	}

	/**
	 * Get provider settings
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $settings Holds provider settings.
	 * @return array
	 */
	public function get_settings( $settings ) {

		$settings['post_id'] = $this->get_the_id();

		return $settings;

	}
}
