<?php
/**
 * Base
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Elements;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Element base class
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Elements\Base
 * @since 1.0.0
 */
class Base extends \OxyEl {

	/**
	 * Element prefix
	 *
	 * @since 1.0.0
	 * @access protected
	 * @var string
	 */
	protected $prefix = 'wpgb';

	/**
	 * Init Element
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function init() {

		// phpcs:ignore WordPress.NamingConventions.ValidVariableName.UsedPropertyNotSnakeCase
		$this->El->useAJAXControls();

	}

	/**
	 * Element slug
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function slug() {

		return $this->prefix . '-' . $this->slug;

	}

	/**
	 * Element icon
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function icon() {

		return WPGB_OXYGEN_URL . 'assets/svg/' . $this->slug . '.svg';

	}

	/**
	 * Element class names
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function class_names() {

		return [];

	}

	/**
	 * Element button place
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function button_place() {

		return 'wpgb::basic';

	}

	/**
	 * Element button priority
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return integer
	 */
	public function button_priority() {

		return 0;

	}

	/**
	 * Render element
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $options Holds element options.
	 */
	public function render( $options ) {

		$this->dequeue_all();
		$this->render_content( $options );

	}

	/**
	 * Dequeue all scripts and styles in editor when rendering
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function dequeue_all() {

		global $wp_scripts, $wp_styles;

		if ( ! defined( 'OXY_ELEMENTS_API_AJAX' ) ) {
			return;
		}

		if ( isset( $wp_styles->queue ) ) {
			$wp_styles->queue  = [];
		}

		if ( isset( $wp_scripts->queue ) ) {
			$wp_scripts->queue  = [];
		}
	}

	/**
	 * Get available grids
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function get_grids() {

		global $wpdb;

		// We only query from Editor ajax request.
		if ( ! is_admin() || ! wp_doing_ajax() || wpgb_doing_ajax() ) {
			return [];
		}

		$grids = $wpdb->get_results( "SELECT id, name, type FROM {$wpdb->prefix}wpgb_grids" );

		return array_column( $grids, 'name', 'id' );

	}

	/**
	 * Get available facets
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function get_facets() {

		global $wpdb;

		// We only query from Editor ajax request.
		if ( ! is_admin() || ! wp_doing_ajax() || wpgb_doing_ajax() ) {
			return [];
		}

		$facets = $wpdb->get_results( "SELECT id, name, type FROM {$wpdb->prefix}wpgb_facets" );

		return array_column( $facets, 'name', 'id' );

	}
}
