<?php
/**
 * Grid element
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Elements;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Grid element
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Elements\Grid
 * @since 1.0.0
 */
final class Grid extends Base {

	/**
	 * Element slug
	 *
	 * @since 1.0.0
	 * @access public
	 * @var string
	 */
	public $slug = 'grid';

	/**
	 * Whether generic CSS has been generated
	 *
	 * @since 1.0.0
	 * @access public
	 * @var boolean
	 */
	public $generic_css = false;

	/**
	 * Element name
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function name() {

		return __( 'Grid', 'wpgb-oxygen' );

	}

	/**
	 * Element controls
	 *
	 * @since 1.0.0
	 * @access public
	 */
	protected function controls() {

		include WPGB_OXYGEN_PATH . 'includes/controls/grid.php';

	}

	/**
	 * Render content
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $options Holds element options.
	 */
	public function render_content( $options ) {

		if ( empty( $options['grid'] ) ) {
			return;
		}

		wpgb_render_grid(
			array_merge(
				[ 'id' => $options['grid'] ],
				$this->default_query( $options ),
				defined( 'OXY_ELEMENTS_API_AJAX' ) ? [ 'inline_style' => true ] : []
			)
		);

		$this->enqueue_grid_assets();

	}

	/**
	 * Get default editor query to simulate main query.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $settings Holds grid settings.
	 * @return array
	 */
	public function default_query( $settings ) {

		$query_args['is_main_query'] = isset( $settings['is_main_query'] ) && 'true' === $settings['is_main_query'];

		if ( ! $query_args['is_main_query'] || ! defined( 'OXY_ELEMENTS_API_AJAX' ) ) {
			return $query_args;
		}

		$post_types = get_post_types( [ 'public' => true ] );
		unset( $post_types['attachment'] );

		return [
			'post_type'   => $post_types,
			'post_status' => 'publish',
		];

	}

	/**
	 * Enqueue element styles in editor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function enqueue_grid_assets() {

		// We do not enqueue on frontend pages.
		if ( ! defined( 'OXY_ELEMENTS_API_AJAX' ) ) {
			return;
		}

		// Enqueue plugin styles.
		wpgb_enqueue_styles();

		// Add inline styles generated by a grid only.
		$styles = wp_styles();
		$styles->print_inline_style( WPGB_SLUG . '-style' );
		$styles->do_item( WPGB_SLUG . '-grids' );
		$styles->do_item( WPGB_SLUG . '-fonts' );

	}

	/**
	 * Set custom CSS to each grid
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array  $options  Holds element settings.
	 * @param string $selector Element CSS selector.
	 * @return string
	 */
	public function customCSS( $options, $selector ) {

		if ( $this->generic_css ) {
			return '';
		}

		$this->generic_css = true;

		return (
			'.oxy-wpgb-grid {' .
				'width: 100%' .
			'}'
		);

	}
}
