<?php
/**
 * Products List
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Providers;

use WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Products_List element
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Providers\Products_List
 * @since 1.0.0
 */
final class Products_List extends Base {

	/**
	 * Get element options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function options() {

		return [
			'itemSelector' => $this->get_selector() . ' > *',
			'pageTitle'    => function_exists( 'woocommerce_page_title' ) ? woocommerce_page_title( false ) : '',
			'isMainQuery'  => (
				empty( $this->element['original']['oxy-woo-products_query_type'] ) ||
				'default' === $this->element['original']['oxy-woo-products_query_type']
			),
		];

	}

	/**
	 * Get inline script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function script() {

		$script  = 'var p=1,o=\'\';template.addEventListener(\'click\',function(e){';
		$script .= 'var t=e.target;p=(t.href||\'\').match(/product-page=(\d+)/)||(t.href||\'\').match(/\/(\d+)/);';
		$script .= '(t.classList.contains(\'page-numbers\')&&t.tagName===\'A\')&&(e.preventDefault()||wpgb.facets.refresh())});';
		$script .= 'window.jQuery&&jQuery(document).ready(function(){jQuery(\'.woocommerce-ordering\').off(\'change\',\'select.orderby\')});';
		$script .= 'template.addEventListener(\'change\',function(e){';
		$script .= 'var t=e.target;(t.classList.contains(\'orderby\')&&t.tagName===\'SELECT\')&&(o=t.value);(e.preventDefault()||wpgb.facets.refresh())});';
		$script .= 'wpgb.facets.on(\'fetch\',function(d){d.paged=(p&&p[1]||1);d.orderby=o;p=1});';
		$script .= 'wpgb.facets.on(\'appended\',function(){var t=this.element.querySelector(\'.page-title\');var a=this.options.pageTitle;t&&a&&(t.textContent=a)});';

		return $script;

	}

	/**
	 * Apply filters for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function before_render() {

		Includes\add_filter_once( 'woocommerce_shortcode_products_query_results', [ $this, 'no_results' ] );
		Includes\add_filter_once( 'shortcode_atts_products', [ $this, 'unset_cache' ] );
		Includes\add_action_once( 'pre_get_posts', [ $this, 'query_args' ] );
		Includes\add_filter_once( 'get_terms', [ $this, 'remove_categories' ] );

		if ( ! empty( $this->element['original']['wpgb_hide_pagination'] ) ) {
			remove_action( 'woocommerce_after_shop_loop', 'woocommerce_pagination' );
		}

		$this->wp_query();

	}

	/**
	 * Filter content output
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $content Element content.
	 * @return string
	 */
	public function after_render( $content = '' ) {

		return $content;

	}

	/**
	 * Unset WooCommerce products query cache to correctly filter
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $out The output array of shortcode attributes.
	 * @return array
	 */
	public function unset_cache( $out ) {

		$out['cache'] = false;

		return $out;

	}

	/**
	 * Add/set arguments in query to allows filtering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function query_args( $query ) {

		$query->set( 'wp_grid_builder', $this->get_id() );

		if ( ! empty( $this->settings['lang'] ) ) {
			$query->set( 'lang', $this->settings['lang'] );
		}

		if ( ! empty( $this->settings['paged'] ) ) {

			$query->set( 'paged', (int) $this->settings['paged'] );
			set_query_var( 'paged', $query->get( 'paged' ) );

		}

		if ( ! empty( $this->settings['orderby'] ) ) {

			// To keep order from WooCommerce sort list.
			$_GET['orderby'] = $this->settings['orderby'];
			( new \WC_Query() )->get_catalog_ordering_args();

		}
	}

	/**
	 * Set and run WordPress query
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function wp_query() {

		global $wp_query;

		if ( ! wpgb_doing_ajax() ) {
			return;
		}

		if ( empty( $this->settings['main_query'] ) ) {
			return;
		}

		if (
			! empty( $this->element['original']['oxy-woo-products_query_type'] ) &&
			'default' !== $this->element['original']['oxy-woo-products_query_type']
		) {
			return;
		}

		// We override the main query when filtering.
		$wp_query = new \WP_Query( $this->settings['main_query'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

	}

	/**
	 * Display no results message if no content found
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param stdClass $results Query results.
	 * @return stdClass
	 */
	public function no_results( $results ) {

		if ( $results->total > 0 ) {
			return $results;
		}

		if ( empty( $this->element['original']['wpgb_noresults'] ) ) {
			return $results;
		}

		echo '<ul class="products">' . wp_kses_post( oxygen_vsb_base64_decode( $this->element['original']['wpgb_noresults'] ) ) . '</ul>';

		return $results;

	}

	/**
	 * Remove categories from shop list if grid is filtered.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $terms Array of found terms.
	 * @return array
	 */
	public function remove_categories( $terms ) {

		return function_exists( 'is_shop' ) && is_shop() && wpgb_has_selected_facets() ? [] : $terms;

	}
}
