<?php
/**
 * Provider
 *
 * @package   WP Grid Builder - Oxygen
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Oxygen\Includes\Providers;

use WP_Grid_Builder_Oxygen\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Default provider
 *
 * @class WP_Grid_Builder_Oxygen\Includes\Providers\Provider
 * @since 1.0.0
 */
class Provider extends Base {

	/**
	 * Get element options
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return array
	 */
	public function options() {

		return [
			'itemSelector' => $this->get_selector() . ' > *',
			'isMainQuery'  => (
				empty( $this->element['original']['wp_query'] ) ||
				'default' === $this->element['original']['wp_query']
			),
		];

	}

	/**
	 * Get inline script
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @return string
	 */
	public function script() {

		$script  = 'var p=1;template.addEventListener(\'click\',function(e){';
		$script .= 'var t=e.target;p=(t.href||\'\').match(/\/(\d+)/);';
		$script .= '(t.classList.contains(\'page-numbers\')&&t.tagName===\'A\')&&(e.preventDefault()||wpgb.facets.refresh())});';
		$script .= 'wpgb.facets.on(\'fetch\',function(d){d.paged=(p&&p[1]||1);p=1});';

		return $script;

	}

	/**
	 * Apply filters for current provider
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function before_render() {

		$this->override_query();

		// To correctly replace the custom query arg (wp_grid_builder) of the main query on initial load.
		Includes\add_action_once( 'pre_get_posts', [ $this, 'query_args' ], ! wpgb_doing_ajax() && is_archive() ? PHP_INT_MAX - 9 : 10 );
		add_action( 'loop_no_results', [ $this, 'no_results' ] );

		$this->wp_query();

	}

	/**
	 * Filter content output
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $content Element content.
	 * @return string
	 */
	public function after_render( $content = '' ) {

		if ( ! empty( $this->element['original']['wpgb_hide_pagination'] ) && method_exists( $this, 'remove_pagination' ) ) {
			$content = $this->remove_pagination( $content );
		}

		remove_action( 'loop_no_results', [ $this, 'no_results' ] );

		return $content;

	}

	/**
	 * Add/set arguments in query to allows filtering
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 */
	public function query_args( $query ) {

		if ( ! $this->is_valid_query( $query ) ) {
			return;
		}

		$query->set( 'wp_grid_builder', $this->get_id() );

		if ( ! empty( $this->settings['lang'] ) ) {
			$query->set( 'lang', $this->settings['lang'] );
		}

		if ( ! empty( $this->settings['paged'] ) ) {

			$query->set( 'paged', (int) $this->settings['paged'] );
			set_query_var( 'paged', $query->get( 'paged' ) );

		}

		if ( ! empty( $this->element['original']['wpgb_hide_pagination'] ) ) {
			$query->set( 'no_found_rows', true );
		}
	}

	/**
	 * Run main WordPress query
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function wp_query() {

		global $wp_query;

		if ( ! wpgb_doing_ajax() ) {
			return;
		}

		if ( empty( $this->settings['main_query'] ) ) {
			return;
		}

		if (
			! empty( $this->element['original']['wp_query'] ) &&
			'default' !== $this->element['original']['wp_query']
		) {
			return;
		}

		// We override the main query when filtering.
		$wp_query = new \WP_Query( $this->settings['main_query'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

	}

	/**
	 * Override query made by Oxygen in dynamic shortcodes (post ID, post title, post type, author, etc.)
	 * Prevent conflicts with advanced queries when filtering with Ajax
	 *
	 * @since 1.0.3
	 * @access public
	 */
	public function override_query() {

		global $oxy_vsb_use_query, $wp_query, $post;

		if ( ! empty( $oxy_vsb_use_query ) || ! wpgb_doing_ajax() ) {
			return;
		}

		// phpcs:disable WordPress.Security.NonceVerification.Recommended
		if ( ! empty( $_GET['wpgb-oxygen-builder'] ) && ! empty( $_GET['post_id'] ) ) {

			$post = get_post( (int) $_GET['post_id'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
			return;

		}
		// phpcs:enable WordPress.Security.NonceVerification.Recommended

		// Override main query if available.
		if ( ! empty( $this->settings['main_query'] ) ) {

			$post = (object) []; // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
			$wp_query = new \WP_Query( $this->settings['main_query'] ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited
			return;

		}

		$post_id = url_to_postid( wp_get_referer() );

		if ( empty( $post_id ) ) {
			return;
		}

		$post = get_post( $post_id ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride.Prohibited

		$oxy_vsb_use_query = new \WP_Query(
			[
				'post_type' => 'any',
				'post__in'  => [ $post_id ],
			]
		);

		// We don't reset post data intentionally.
		if ( $oxy_vsb_use_query->have_posts() ) {
			$oxy_vsb_use_query->the_post();
		}
	}

	/**
	 * Check if it is a valid advanced query (advanced query with ACF/Meta Box shortcodes)
	 *
	 * @since 1.0.6
	 * @access public
	 *
	 * @param WP_Query $query The WP_Query instance.
	 * @return boolean
	 */
	public function is_valid_query( $query ) {

		global $wp_query;

		$is_advanced_query = ! empty( $this->element['original']['wp_query'] ) && 'advanced' === $this->element['original']['wp_query'];
		$is_field_query    = in_array( ( (array) $query->get( 'post_type', '' ) )[0], [ 'meta-box', 'acf-field', 'acf-field-group' ], true );
		$is_same_query     = ! wpgb_doing_ajax() && $wp_query->query_vars === $query->query_vars;

		if ( ! $is_advanced_query || ( ! $is_field_query && ! $is_same_query ) ) {
			return true;
		}

		// Watch next query until we match the Repeater query.
		Includes\add_action_once( 'pre_get_posts', [ $this, 'query_args' ] );

		return false;

	}

	/**
	 * Display no results message if no content found
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function no_results() {

		if ( empty( $this->element['original']['wpgb_noresults'] ) ) {
			return;
		}

		echo '<div>' . wp_kses_post( oxygen_vsb_base64_decode( $this->element['original']['wpgb_noresults'] ) ) . '</div>';

	}
}
