<?php
/**
 * Helpers
 *
 * @package   WP Grid Builder - Pods
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Pods\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Helpers
 *
 * @class WP_Grid_Builder_Pods\Includes\Helpers
 * @since 1.0.0
 */
trait Helpers {

	/**
	 * Holds pods instances
	 *
	 * @since 1.0.0
	 * @access private
	 * @var array
	 */
	private $pods = [];

	/**
	 * Holds field names and associated field types
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var array
	 */
	private $field_types = [
		'paragraph' => 'paragraph_field',
		'wysiwyg'   => 'paragraph_field',
		'code'      => 'paragraph_field',
		'pick'      => 'relationship_field',
		'datetime'  => 'date_field',
		'date'      => 'date_field',
		'file'      => 'file_field',
		'boolean'   => 'toggle_field',
	];

	/**
	 * Check if it is a Pods field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $field_id Field identifier.
	 * @return boolean
	 */
	public function is_pods_field( $field_id ) {

		return count( explode( 'pods/', $field_id ) ) > 1;

	}

	/**
	 * Get Pods field id
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $field_id Field identifier.
	 * @return string
	 */
	public function get_pods_field_id( $field_id ) {

		return explode( 'pods/', $field_id )[1];

	}

	/**
	 * Get Pods field type
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $field_id Field identifier.
	 * @return string
	 */
	public function get_pods_field_type( $field_id ) {

		return explode( '/', $this->get_pods_field_id( $field_id ) )[0];

	}

	/**
	 * Get Pods field name
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $field_id Field identifier.
	 * @return string
	 */
	public function get_pods_field_name( $field_id ) {

		return explode( '/', $this->get_pods_field_id( $field_id ) )[1];

	}

	/**
	 * Get Pods field settings
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string  $field_id  Field identifier.
	 * @param integer $object_id Object id to index.
	 * @return array
	 */
	public function get_pods_field_settings( $field_id, $object_id ) {

		$pods = $this->get_pods_field( $field_id, $object_id );

		if ( ! $pods ) {
			return [];
		}

		return $pods->fields( $this->get_pods_field_name( $field_id ) );

	}

	/**
	 * Get Pods instance
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string  $field_id  Field identifier.
	 * @param integer $object_id Object id to index.
	 * @return boolean|object
	 */
	public function get_pods_field( $field_id, $object_id ) {

		$field = $this->get_pods_field_type( $field_id );
		$cache = $field . '/' . $object_id;

		if ( ! empty( $this->pods[ $cache ] ) ) {
			return $this->pods[ $cache ];
		}

		$pods = pods( $field, $object_id );

		if ( ! $pods || ! $pods->valid() || ! $pods->exists() ) {
			return false;
		}

		$this->pods[ $cache ] = $pods;

		return $pods;

	}

	/**
	 * Get field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string  $field_id  Field identifier.
	 * @param integer $object_id Object id to index.
	 * @param boolean $display   Whether to get display value.
	 * @return string|array
	 */
	public function get_field( $field_id, $object_id, $display = true ) {

		$pods = $this->get_pods_field( $field_id, $object_id );

		if ( ! $pods ) {
			return '';
		}

		$field = $this->get_pods_field_settings( $field_id, $object_id );

		if ( empty( $field ) ) {
			return '';
		}

		$value = $this->get_field_value( $field, $pods, $display );
		$value = array_map(
			function( $value ) use ( $field, $pods ) {
				return $this->format_field_value( $value, $field, $pods );
			},
			$value
		);

		return count( $value ) > 1 || ! $display ? $value : current( $value );

	}

	/**
	 * Get field value
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array   $field   Field settings.
	 * @param object  $pods    Pods instance.
	 * @param boolean $display Whether to get display value.
	 * @return array
	 */
	public function get_field_value( $field, $pods, $display ) {

		if ( $display ) {
			return (array) $pods->display( $field['name'] );
		}

		$params = [
			'name'   => $field['name'],
			'keyed'  => true,
			'output' => 'names',
		];

		if ( 'pick' === $field['type'] && in_array( $field['pick_object'], [ 'taxonomy', 'post_format' ], true ) ) {
			$params['output'] = 'arrays';
		}

		return array_filter(
			(array) $pods->field( $params ),
			function( $value ) {
				return ! ( ! is_array( $value ) && ( is_null( $value ) || '' === trim( $value ) ) );
			}
		);
	}

	/**
	 * Format field value
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param mixed  $value Field value.
	 * @param array  $field Holds field settings.
	 * @param object $pods  Pods instance.
	 * @return array
	 */
	public function format_field_value( $value, $field, $pods ) {

		$field_method = 'default_field';

		if (
			isset( $this->field_types[ $field['type'] ] ) &&
			method_exists( $this, $this->field_types[ $field['type'] ] )
		) {
			$field_method = $this->field_types[ $field['type'] ];
		}

		if ( method_exists( $this, $field_method ) ) {
			return call_user_func( [ $this, $field_method ], $value, $field, $pods );
		}

		return [];

	}
}
