<?php
/**
 * Indexer
 *
 * @package   WP Grid Builder - Pods
 * @author    Loïc Blascos
 * @copyright 2019-2022 Loïc Blascos
 */

namespace WP_Grid_Builder_Pods\Includes;

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Indexer
 *
 * @class WP_Grid_Builder_Pods\Includes\Indexer
 * @since 1.0.0
 */
final class Indexer {

	use Helpers;

	/**
	 * Constructor
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function __construct() {

		add_filter( 'wp_grid_builder/indexer/index_object', [ $this, 'maybe_handle' ], 10, 3 );

	}

	/**
	 * Maybe index Pods field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param array $rows      Holds rows to index.
	 * @param array $object_id Object id to index.
	 * @param array $facet     Holds facet settings.
	 * @return array
	 */
	public function maybe_handle( $rows, $object_id, $facet ) {

		if ( ! $this->is_pods_field( $facet['source'] ) ) {
			return $rows;
		}

		return $this->get_field( $facet['source'], $object_id, false );

	}

	/**
	 * Handle paragraph field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $value Field value.
	 * @param array  $field Holds field settings.
	 * @return array
	 */
	public function paragraph_field( $value, $field ) {

		$value = trim( wp_strip_all_tags( $value, true ) );

		return [
			'facet_value' => $value,
			'facet_name'  => $value,
		];
	}

	/**
	 * Handle toggle field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param integer $value Field value.
	 * @param array   $field Holds field settings.
	 * @return array
	 */
	public function toggle_field( $value, $field ) {

		if ( ! is_scalar( $value ) || '' === $value ) {
			return [];
		}

		if ( empty( $field['boolean_yes_label'] ) ) {
			$field['boolean_yes_label'] = __( 'Yes', 'wpgb-pods' );
		}

		if ( empty( $field['boolean_no_label'] ) ) {
			$field['boolean_no_label'] = __( 'No', 'wpgb-pods' );
		}

		return [
			'facet_value' => $value,
			'facet_name'  => (int) $value ? $field['boolean_yes_label'] : $field['boolean_no_label'],
		];
	}

	/**
	 * Handle date field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param mixed $value Field value.
	 * @param array $field Holds field settings.
	 * @return array
	 */
	public function date_field( $value, $field ) {

		if (
			empty( $value ) ||
			'0000-00-00' === $value ||
			'0000-00-00 00:00:00' === $value
		) {
			return [];
		}

		return [
			'facet_value' => $value,
			'facet_name'  => $value,
		];
	}

	/**
	 * Handle file field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param mixed $value Field value.
	 * @param array $field Holds field settings.
	 * @return array
	 */
	public function file_field( $value, $field ) {

		if ( empty( $value ) ) {
			return [];
		}

		return [
			'facet_value' => $value,
			'facet_name'  => $value,
		];
	}

	/**
	 * Handle relationship field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param mixed  $value Field value.
	 * @param array  $field Holds field settings.
	 * @param object $pods  Pods instance.
	 * @return array
	 */
	public function relationship_field( $value, $field, $pods ) {

		if ( empty( $value ) ) {
			return [];
		}

		switch ( $field['pick_object'] ) {
			case 'user':
			case 'media':
			case 'comments':
			case 'nav_menu':
			case 'post_type':
				return [
					'facet_value' => $value,
					'facet_name'  => $value,
				];
			case 'taxonomy':
			case 'post_format':
				if ( empty( $value['term_id'] ) ) {
					return [];
				}

				return [
					'facet_value'  => $value['slug'],
					'facet_name'   => $value['name'],
					'facet_id'     => $value['term_id'],
					'facet_parent' => $value['parent'],
					'facet_order'  => $value['term_order'],
				];
			default:
				$data = $pods->fields( $field['name'], 'data' );
				return [
					'facet_value' => $value,
					'facet_name'  => ! empty( $data[ $value ] ) ? $data[ $value ] : $value,
				];
		}
	}

	/**
	 * Handle default field
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 * @param string $value Field value.
	 * @return array
	 */
	public function default_field( $value ) {

		return [
			'facet_value' => $value,
			'facet_name'  => $value,
		];
	}
}
